#ifndef _FCRYPT_H
#define _FCRYPT_H

#define DES_BLOCK_SIZE	8
#define	DES_KEY_SIZE	8

#define	DES3_BLOCK_SIZE	DES_BLOCK_SIZE
#define	DES3_KEY_SIZE	(3 * DES_KEY_SIZE)

struct fcrypt_3des_ctx {
	uint8_t iv[DES_BLOCK_SIZE];
	uint8_t key[DES3_KEY_SIZE];
};

/* Values for status in fcrypt_dev */
#define BOARD_FREE 0
#define BOARD_BUSY 1
#define BOARD_CRYPT_OK 2
#define BOARD_WRONG_INT 3

struct fcrypt_dev {
	struct pci_dev *pdev;
	long opreg,
	     adrctr,
	     aport,
	     cport;
	void *ipage, *opage;
	unsigned long ipage_phys, opage_phys;
	wait_queue_head_t *pwaitq;
	volatile unsigned char status;
};

/* Symbolic constants for accessing various FastCrypt functions. */

//
// PCI-Chip Address-offsets (from PortBase0)
//
#define FIFO_REG_OFFSET         0x20       // FIFO Register (B-Port)
#define MWAR_REG_OFFSET         0x24       // Master Write Address Register
#define MWTC_REG_OFFSET         0x28       // Master Write Transfer Count Reg.
#define MRAR_REG_OFFSET         0x2C       // Master Read Adr. Reg.
#define MRTC_REG_OFFSET         0x30       // Master Read Transfer Count Reg.
#define MCSR_REG_OFFSET         0x3C       // BusMaster Control/Status Register
#define INTR_CTRL_REG_OFFSET    0x38       // Interrupt Control/Status Register
//
// PCI-Chip Commands (for MCSR-Reg)
//
#define PCI_TO_ADDON_RESET      0x02000000  // Reset PCI to Addon Flags
#define ADDON_TO_PCI_RESET      0x04000000  // Reset Addon to PCI Flags
#define PCI_RESET_CMD           0x0E000000  // Reset FIFO/Mailbox Flags
#define PCI_FULL_RESET_CMD      0x0F000000  // Reset FIFO/Mailbox & SuperCrypt
#define PCI_BUSMASTER_WRITE_ENABLE 0x00000400 //Enable Busmaster write transfer
#define PCI_BUSMASTER_READ_ENABLE  0x00004000 //Enable Busmaster read transfer
//
// PCI-Chip FIFO-Status Bitmasks (for MCSR-Reg)
//
#define FIFO_WRITE_FULL         0x00000001  // PCI to FIFO->AddOn full
#define FIFO_WRITE_HALF         0x00000002  // PCI to FIFO->AddOn half empty
#define FIFO_WRITE_EMPTY        0x00000004  // PCI to FIFO->AddOn empty
#define FIFO_READ_FULL          0x00000008  // AddOn->PCI FIFO full
#define FIFO_READ_HALF          0x00000010  // AddOn->PCI FIFO half full
#define FIFO_READ_EMPTY         0x00000020  // AddOn->PCI FIFO empty
//
// PCI-Chip Interrupt selection (INTCSR-Reg)
//
#define INT_ON_WRITE_COMPLETE   0x00004000 //initiate an Int. on write complete
#define INT_ON_READ_COMPLETE    0x00008000 //initiate an Int. on read complete
//
// PCI-Chip Actual Interrupt Bitmasks
//
#define INT_ASSERTED            0x00800000  // Interrupt-Condition
#define INT_TARGET_ABORT        0x00200000  // Target-Abort while Busmaster-DMA
#define INT_MASTER_ABORT        0x00100000  // Master-Abort while Busmaster-DMA
#define READ_TRANSFER_COMPLETE  0x00080000  /* completition of PCI-Master
                                             * Operation */
#define WRITE_TRANSFER_COMPLETE 0x00040000  /* completition of PCI-Master
                                             * Operation */
//
// AdrCtrl-Reg. Bitmasks
//
#define AdrCtl_EBT_BIT          0x80000000  // Enable B-Port-Transfer
#define AdrCtl_CIE_BIT          0x08000000  // Crypt Input-Buffer empty
#define AdrCtl_COF_BIT          0x10000000  // Crypt Output-Buffer full

//
// A-Port Address-register (from PortBase2)
//
#define A_PORT_CXA_AXC_OFFSET   0x02        // Data Exchange to/from C-Port
#define A_PORT_ASTAT_OFFSET     0x03        // Status Register of A-Port (read)
#define A_PORT_APTR_OFFSET      0x03        // Addresspointer of A-Port (write)
#define A_PORT_AIAR_OFFSET      0x04        // Indirect addressed register
#define A_PORT_ADATA_OFFSET     0x08        // Crypt-data in/out    
//
// A_Port indirect-registers
//
#define A_PORT_CFG_IOFFSET      0x00  // Port Configuration Register (r/part.w)
#define A_PORT_AIE_IOFFSET      0x01  // Interrupt-Enable A-Port (write)
#define A_PORT_AIS_IOFFSET      0x01  // Interrupt-Source A-Port (read)
#define A_PORT_DMA_IOFFSET      0x02  // DMA-Mode (r/w)
#define A_PORT_IV_IOFFSET       0x03  // Initial-Vector (write)
#define A_PORT_CRL_IOFFSET      0x03  // Chip Revision Level (read)
#define A_PORT_DBC_IOFFSET      0x0B  // Data Block Counter (r/w)
#define A_PORT_KYE_IOFFSET      0x0E  // Key Enable Register (read)
#define A_PORT_KPTR_IOFFSET     0x10  // Key Pointer Register (r/w)
#define A_PORT_MOD_IOFFSET      0x13  // Cipher Mode Register (r/w)
#define A_PORT_CMD_IOFFSET      0x14  // A-Port Command (write)
#define A_PORT_MID_IOFFSET      0x15  // Multitasking ID (r/w)
#define A_PORT_CID_IOFFSET      0x19  // C-Port ID (read)
#define A_PORT_XEN_IOFFSET      0x1D  // Extended enable Register (read)
//
// A-Port Command-Codes
//
#define A_PORT_StartCipher_CMD  0x01  // starting the Cipher engine
#define A_PORT_StopCipher_CMD   0x00  // stop and reset buffers
#define A_PORT_Reset_CMD        0x08  // cleasr all registers except the IV
#define A_PORT_StopStoreIV1_CMD 0x04  // stops and stores act. IV as IV#1
#define A_PORT_Fetch1Start_CMD  0x05  // loads IV#1 and starts cipher
#define A_PORT_StopStoreIV2_CMD 0x06  // stops and stores act. IV as IV#2
#define A_PORT_Fetch2Start_CMD  0x07  // loads IV#2 and starts cipher
//
// DMA-Bitmasks
//
#define DMA_CH1_READ            0x00  // write through ch. 2
#define DMA_CH1_WRITE           0x01  // read through ch. 2
#define DMA_ENABLE_CH1          0x02  // enable DMA-Channel 1
#define DMA_ENABLE_CH2          0x08  // enable DMA-Channel 2
#define DMA_BURST_CH1           0x04  // enable Burst-mode on Ch.1
#define DMA_BURST_CH2           0x10  // enable Burst-mode on Ch.2
#define DMA_ENABLE          DMA_CH1_WRITE + DMA_ENABLE_CH1 + DMA_ENABLE_CH2 \
                            + DMA_BURST_CH1 + DMA_BURST_CH2
//
// Interrupt-Enable-Bitmasks
//
#define AIE_KEY_PROTECT  0x01  // Interrupt on setting a pointer to invalid key
#define AIE_INPUT_EMPTY  0x02  // Int. while empty input buffer
#define AIE_OUTPUT_FULL  0x04  // Int. while output buffer full
#define AIE_DBC_ZERO     0x08  // DataBlockCounter is zero
#define AIE_CXA_FULL     0x10  // Message from C-Port to A-Port is pending
//
// Cipher-modes
//
#define ENCODE                  0x00  // Encrypt-Mode
#define DECODE                  0x01  // Decrypt-Mode
#define	ENCDEC_MASK		0x01

#define TRIPLE_CIPHER           0x02  // Triple-Cipher Mode
#define SINGLE_KEY_LENGTH       0x00  // Single Key (56 bit)
#define EXTENDED_KEY_LENGTH     0x04  // Double key (112 bit)
#define ECB_MODE                0x00
#define CBC_MODE                0x10
#define OFB_MODE                0x20
#define CFB_MODE                0x30
#define KSG_MODE                0x40
#define MAC_MODE                0x50
#define CRYPT_MODE_MASK         0xf0

//
// ASTAT-Bitmasks
//
#define ASTAT_CIPHER_ACTIVE     0x01  // Cipher engine busy or idle ?
#define ASTAT_OUTPUT_EMPTY      0x04  // A-Port Output empty
#define ASTAT_INPUT_FULL        0x02  // A-Port Input full

//
// C-Port Address-register (from PortBase3)
//
#define C_PORT_AXC_CXA_OFFSET   0x00  // no offset for ExchangeReg. to A-Port
#define C_PORT_CSTAT_OFFSET     0x01  // Status Register of C-Port (read)
#define C_PORT_CPTR_OFFSET      0x01  // Addresspointer of C-Port (write)
#define C_PORT_CIAR_OFFSET      0x02  // indirect C-Port Register
#define C_PORT_CIB_COB_OFFSET   0x04  // Crypt-data in/out
//
// C-Port indirect-registers
//
#define C_PORT_CFG_IOFFSET      0x00  /* Port Configuration Register
                                       * (same as A-Port) (r/w) */
#define C_PORT_CIE_IOFFSET      0x01  // Interrupt-Enable C-Port (write)
#define C_PORT_CIS_IOFFSET      0x01  // Interrupt-Source C-Port (read)
#define C_PORT_KYE_IOFFSET      0x04  // 2 byte(!) Key-enable Register (write)
#define C_PORT_CMD1_IOFFSET     0x05  // Command-Reg. 1 (key and Alg. loading)
#define C_PORT_CMD2_IOFFSET     0x06  // Command-Reg. 2 ( -"- )
#define C_PORT_CDP_IOFFSET      0x07  // C-Port Data-Path (for X-Box & Keys)
#define C_PORT_XEN_IOFFSET      0x08  /* ExtendedEnableReg.
                                       * (access-rights for A-Port) */
//
// C-Port Command-words for Command-Reg. 1
//
#define C_PORT_Reset_CMD1       0x08  // C-Port reset
//
// C-Port Command-words for Command-Reg. 2
//
#define C_PORT_XBoxLoad_CMD2    0x08  // prepare for X-Box load
#define C_PORT_ExtKey_CMD2      0x04  // use Extended Key (112 Bit)
//
// CSTAT-Bitmasks
//
#define CSTAT_CIPHER_ACTIVE     0x01  // Cipher is active
#define CSTAT_KEY_WRITE_PROTECT 0x02  // Key write-protection error
#define CSTAT_AXC_FULL          0x04  // there's a message from A-Port waiting
#define CSTAT_CXA_NOT_EMPTY     0x08  // there's a message to A-Port pending
#define CSTAT_MASTER_RESET      0x10  // Master-Reset occured
#define CSTAT_KEY_PARITY_ERROR  0x20  // Parity error during key upload
//
// CFG-Bitmasks (valid for A-Port, too)
//
#define CFG_ENABLE_A_PORT       0x80  // enable A-Port
#define CFG_ENABLE_A_PORT_IV    0x40  // enable IV-load for A-Port
#define CFG_ENABLE_B_READ       0x01  // enable B-Port Read Access
#define CFG_ENABLE_B_WRITE      0x02  // enable B-Port Write Access
#define CFG_ENABLE_B_RW         0x03  // enable B-Port r/w Access
#define CFG_ENABLE_B_DBM        0x06  // enable DBM with B-Port

#endif /* _FCRYPT_H */
